import React, { ReactNode, useRef, useEffect, useState } from 'react';
import { createPortal } from 'react-dom';
import { FaTimes } from 'react-icons/fa';
import { disableBodyScroll, enableBodyScroll, clearAllBodyScrollLocks } from 'body-scroll-lock';
import FocusTrap from 'focus-trap-react';
import styles from '@/styles/common/Modal.module.scss';

export interface ModalProps {
  isOpen: boolean;
  onClose: () => void;
  title?: string;
  size?: 'sm' | 'md' | 'lg' | 'xl' | 'full';
  children: ReactNode;
  showCloseButton?: boolean;
  closeOnOverlayClick?: boolean;
  closeOnEsc?: boolean;
  className?: string;
  testId?: string;
  initialFocus?: boolean | string | HTMLElement;
}

const Modal: React.FC<ModalProps> = ({
  isOpen,
  onClose,
  title,
  size = 'md',
  children,
  showCloseButton = true,
  closeOnOverlayClick = true,
  closeOnEsc = true,
  className,
  testId,
  initialFocus,
}) => {
  const [mounted, setMounted] = useState(false);
  const modalRef = useRef<HTMLDivElement>(null);
  
  useEffect(() => {
    setMounted(true);
    
    return () => setMounted(false);
  }, []);
  
  useEffect(() => {
    // Handle body scroll lock
    if (isOpen && modalRef.current) {
      disableBodyScroll(modalRef.current);
    } else if (modalRef.current) {
      enableBodyScroll(modalRef.current);
    }
    
    // Clean up on unmount
    return () => clearAllBodyScrollLocks();
  }, [isOpen]);
  
  useEffect(() => {
    // Handle escape key press
    const handleEscapeKey = (e: KeyboardEvent) => {
      if (closeOnEsc && e.key === 'Escape' && isOpen) {
        onClose();
      }
    };
    
    if (closeOnEsc) {
      document.addEventListener('keydown', handleEscapeKey);
    }
    
    return () => {
      if (closeOnEsc) {
        document.removeEventListener('keydown', handleEscapeKey);
      }
    };
  }, [closeOnEsc, isOpen, onClose]);
  
  const handleOverlayClick = (e: React.MouseEvent) => {
    if (closeOnOverlayClick && e.target === e.currentTarget) {
      onClose();
    }
  };
  
  const modalContent = (
    <FocusTrap
      active={isOpen}
      focusTrapOptions={{
        initialFocus: initialFocus || undefined,
        escapeDeactivates: closeOnEsc,
        clickOutsideDeactivates: closeOnOverlayClick,
      }}
    >
      <div
        className={`${styles.overlay} ${isOpen ? styles.active : ''}`}
        onClick={handleOverlayClick}
        aria-hidden={!isOpen}
        data-testid={testId && `${testId}-overlay`}
      >
        <div
          ref={modalRef}
          className={`${styles.modal} ${styles[`modal--${size}`]} ${className || ''}`}
          role="dialog"
          aria-modal="true"
          aria-labelledby={title ? 'modal-title' : undefined}
          data-testid={testId}
        >
          {(title || showCloseButton) && (
            <header className={styles.header}>
              {title && (
                <h2 className={styles.title} id="modal-title">
                  {title}
                </h2>
              )}
              {showCloseButton && (
                <button
                  type="button"
                  className={styles.closeButton}
                  onClick={onClose}
                  aria-label="Close modal"
                  data-testid={`${testId}-close-button`}
                >
                  <FaTimes />
                </button>
              )}
            </header>
          )}
          
          <div className={styles.content}>
            {children}
          </div>
        </div>
      </div>
    </FocusTrap>
  );
  
  // Portal modal to document.body for proper stacking context
  return mounted && isOpen ? createPortal(modalContent, document.body) : null;
};

export default Modal;